import {defineStore} from 'pinia';
import {ref} from 'vue';


export interface ApplicationError {
  message: string;
  title?: string;
  unRecoverable?: boolean;
}

const errorsEqual = (a: ApplicationError, b: ApplicationError): boolean => a.message === b.message && a.title === b.title && a.unRecoverable === b.unRecoverable;

export const useErrorStore = defineStore('error', () => {

  const activeError = ref<ApplicationError | null>(null);

  const errorQueue: ApplicationError[] = [];

  const _updateActiveError = (): void => {
    if (errorQueue.length === 0) {
      activeError.value = null;
    } else if (activeError.value !== errorQueue[0]) {
      activeError.value = errorQueue[0] as ApplicationError;
    }
  };

  const push = (error: ApplicationError): void => {
    if (errorQueue.find(existingError => errorsEqual(existingError, error))) {
      // This exact error is already in queue -> skip
      return;
    }
    errorQueue.push(error);
    _updateActiveError();
  };

  const clear = (message: string): void => {
    const idx = errorQueue.findIndex(error => error.message === message);
    if (idx < 0) {
      return;
    }
    errorQueue.splice(idx, 1);
    _updateActiveError();
  };

  const clearActiveError = (): void => {
    if (errorQueue.length === 0) {
      return;
    }
    errorQueue.splice(0, 1);
    _updateActiveError();
  };

  return {
    activeError,
    push,
    clear,
    clearActiveError,
  };
});
